/*
 * MAX7219制御ライブラリー
 * 
 * Copyright (C) 2019 Picosoft Co., Ltd.
 */
#include <xc.h>
#include "Max7219.h"

#define P_CLK 14
#define P_LOAD 15
#define P_DIN 16

static void delay() {
}

static void out(int port, bool sw) {
	switch (port) {
		case 14: { // RB5
			TRISBbits.TRISB5 = false;
			LATBbits.LATB5 = sw;
			break;
		}
		case 15: { // RB6
			TRISBbits.TRISB6 = false;
			LATBbits.LATB6 = sw;
			break;
		}
		case 16: { // RB7
			TRISBbits.TRISB7 = false;
			LATBbits.LATB7 = sw;
			break;
		}
		default: { // 未実装
			break;
		}
	}
}

static void shiftOut(int dataPort, int clockPort, bool isLsbFirst, int value) {
	for (int i = 0; i < 8; i++) {
		out(dataPort, (isLsbFirst) ? value & (1 << i) : value & (1 << (7 - i)));
		out(clockPort, true);
		out(clockPort, false);
	}
}

// コマンドを発行します。
void max7219(int addr, int data) {
	out(P_LOAD, false);
	shiftOut(P_DIN, P_CLK, false, addr);
	shiftOut(P_DIN, P_CLK, false, data);
	out(P_LOAD, true);
	delay();
}

// テストモード(false:ノーマル、true:テスト)を設定します。
void setDisplayTest(bool mode) {
	max7219(15, mode);
}

// デコードモード(0xff:デコード、0:ビット指定)を設定します。
void setDecode(int mode) {
	max7219(9, mode);
}

// 輝度(0～15)を設定します。
void setIntensity(int intensity) {
	max7219(10, intensity);
}

// 表示桁数(0～7で1桁～8桁)を設定します。
void setScanLimit(int limit) {
	max7219(11, limit);
}

// 表示をON/OFFします。
void shutdown(bool sw) {
	max7219(12, !sw);
}

// 初期化します。
void max7219Init() {
	out(P_LOAD, true);
	out(P_DIN, false);
	out(P_CLK, false);
	delay();
	setDisplayTest(false);
	setDecode(0);
	setIntensity(2);
	setScanLimit(7);
	shutdown(false);
}
