/*
   This program is provided under the LGPL license ver 2.
   Written by Katsumi.
   http://hp.vector.co.jp/authors/VA016157/
   kmorimatsu@users.sourceforge.jp
*/

/*	Environment(machine)-specific basic functions follow
	These are specified to access to MZ-80K (with 28 KB RAM) memory map.
	I/O ports are connected to PORTB of PIC32MX150F128B.
	To emulate the other computer, these must be modified.
*/

// _getCode() macro (as inline function) that fetches a code using PC, and increment PC.
#define _getCode() (\
	(regPC<0x0300) ? ROM[regPC++] :(\
	(0x7FFF<regPC && regPC<0x9800) ? RAM[(regPC++)&0x1FFF] : 0xFF\
	))
//#define getCode() _getCode()

UINT8 getCode(){
	// Using getCode() as inline function for all will use much memory to exceed the maximum.
	return _getCode();
}
//*/

unsigned char readMemory(UINT16 addr){
	addr&=0xffff;
	if (addr<0x0300) {
		return ROM[addr];
	} else if (addr<0x8000) {
		return 0xFF; // RST 38
	} else if (addr<0x9800) {
		return RAM[addr&0x1FFF];
	} else {
		return 0xFF; // RST 38
	}
}

void writeMemory(UINT16 addr, UINT8 data){
	addr&=0xffff;
	data&=0xff;
	if (addr<0x8000) {
		return;
	} else if (addr<0x9800) {
		RAM[addr&0x1FFF]=data;
	} else {
		return;
	}
}

// Global 8255 values
UINT8 g_porta, g_portb, g_portc;

// readIO() function
unsigned char readIO(UINT8 addrL, UINT8 addrH){
	// Key input support (Port A)
	switch (addrL) {
		case 0xF8:
			return g_porta;
		case 0xF9:
			return g_portb;
		case 0xFA:
			return g_portc;
		default:
			return 0xFF;
	}
}

// 2019-02-22 picosoft -----(S)
#include <nvm.h>
#include <string.h>
#define PGM_END 0x9d040000 // PIC32MX170F256B
#define SAVE_AREA (PGM_END - 1024)
#define TK80_MON_WORK_SIZE (0x83ff - 0x83c7 + 1)

static void flashLoad() {
	char monitorSave[TK80_MON_WORK_SIZE];
	memmove(monitorSave, &RAM[0x3c7], TK80_MON_WORK_SIZE);
	memmove(RAM, (const void*)SAVE_AREA, 1024);
	memmove(&RAM[0x3c7], monitorSave, TK80_MON_WORK_SIZE);
}

static void flashSave() {
	NVMErasePage((void*)SAVE_AREA);
	int i;
	for (i = 0; i < 8; i++) { // 128 x 8 = 1024
		NVMWriteRow((void*)(SAVE_AREA + i * 128), &RAM[i * 128]);
	}
}
// 2019-02-22 picosoft -----(E)

// writeIO() function
void writeIO(UINT8 addrL, UINT8 addrH, UINT8 data){

// 2019-05-14 picosoft -----(S)
	if (addrL != 0x9e) {
		if ((addrL & 0x03) == 0x02) { // Port C
			TRISAbits.TRISA4 = 0;
			if (data & 0x02) { // bit1: Speaker
				LATAbits.LATA4 = 1;
			} else {
				LATAbits.LATA4 = 0;
			}

		}
	}
// 2019-05-14 picosoft -----(E)

	// Key input support
	switch (addrL) {
		case 0x9e: // Event handler
			switch (data) {
				case 0x00: // STORE DATA
					// 2019-02-22 picosoft -----(S)
					flashSave();
					// 2019-02-22 picosoft -----(E)
					return;
				case 0x01: // LOAD DATA
					// 2019-02-22 picosoft -----(S)
					flashLoad();
					// 2019-02-22 picosoft -----(E)
					return;
				default:
					return;
			}
		case 0xfa: // Set row in key matrix (TK-80)
			g_portc=data;
			g_porta=0xff;
			if (!(data&0x10)) g_porta&=g_keymatrix[0];
			if (!(data&0x20)) g_porta&=g_keymatrix[1];
			if (!(data&0x40)) g_porta&=g_keymatrix[2];
			return;
		case 0xfb: // 8255 control (TK-80)
			if (data&0x80) return; // Ignore mode change.
			data&=0x0f; // Bit change in Port C follows.
			g_portc&=(1<<(data>>1))^0xff;
			g_portc|=(data&0x01)<<(data>>1);
			data=g_portc;
			g_porta=0xff;
			if (!(data&0x10)) g_porta&=g_keymatrix[0];
			if (!(data&0x20)) g_porta&=g_keymatrix[1];
			if (!(data&0x40)) g_porta&=g_keymatrix[2];
			return;
		default:
			return;
	}
}

void triggerInt(){
	if (0x8000<=z80PC) intZ80(0xFF);
}

/*	End of environment-specific basic functions */
